Package: InteractiveComplexHeatmap
Type: Package
Title: Make Interactive Complex Heatmaps
Version: 1.20.0
Date: 2025-01-30
Authors@R: person("Zuguang", "Gu", email = "guzuguang@suat-sz.edu.cn", role = c("aut", "cre"),
                  comment = c('ORCID'="0000-0002-7395-8709"))
Depends: R (>= 4.0.0), ComplexHeatmap (>= 2.11.0)
Imports: grDevices, stats, shiny, grid, GetoptLong, S4Vectors (>=
        0.26.1), digest, IRanges, kableExtra (>= 1.3.1), utils,
        svglite, htmltools, clisymbols, jsonlite, RColorBrewer,
        fontawesome
Suggests: knitr, rmarkdown, testthat, EnrichedHeatmap, GenomicRanges,
        data.table, circlize, GenomicFeatures, tidyverse, tidyHeatmap,
        cluster, org.Hs.eg.db, simplifyEnrichment, GO.db, SC3,
        GOexpress, SingleCellExperiment, scater, gplots, pheatmap,
        airway, DESeq2, DT, cola, BiocManager, gridtext, HilbertCurve
        (>= 1.21.1), shinydashboard, SummarizedExperiment, pkgndep, ks
VignetteBuilder: knitr
Description: This package can easily make heatmaps which are produced 
    by the ComplexHeatmap package into interactive applications. It provides two types of interactivities: 
    1. on the interactive graphics device, and 2. on a Shiny app. It also provides 
    functions for integrating the interactive heatmap widgets for more complex Shiny app development.
biocViews: Software, Visualization, Sequencing
URL: https://github.com/jokergoo/InteractiveComplexHeatmap
BugReports: https://github.com/jokergoo/InteractiveComplexHeatmap/issues
License: MIT + file LICENSE
git_url:
        https://git.bioconductor.org/packages/InteractiveComplexHeatmap
git_branch: RELEASE_3_23
git_last_commit: 9ffb2b9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:46:14 UTC; biocbuild
Author: Zuguang Gu [aut, cre] (ORCID: <https://orcid.org/0000-0002-7395-8709>)
Maintainer: Zuguang Gu <guzuguang@suat-sz.edu.cn>
Built: R 4.6.1; ; 2026-08-02 07:47:06 UTC; unix
