S3method("interactivate", "DESeqDataSet")
export("interactivate.DESeqDataSet")
S3method("interactivate", "kde")
export("interactivate.kde")
export("HeatmapInfoOutput")
export("InteractiveComplexHeatmapModal")
export("InteractiveComplexHeatmapOutput")
export("InteractiveComplexHeatmapWidget")
export("all_column_indices")
export("all_row_indices")
export("getPositionFromBrush")
export("getPositionFromClick")
export("getPositionFromDblclick")
export("getPositionFromHover")
export("htPositionsOnDevice")
export("htShiny")
export("htShinyExample")
export("ht_shiny")
export("interactivate")
export("interactivateDensity2D")
export("is_in_sub_heatmap")
export("makeInteractiveComplexHeatmap")
export("originalHeatmapOutput")
export("record_observation")
export("selectArea")
export("selectPosition")
export("subHeatmapOutput")

import(ComplexHeatmap)
import(GetoptLong)
import(IRanges)
import(RColorBrewer)
import(S4Vectors)
import(clisymbols)
import(digest)
import(fontawesome)
import(grid)
import(shiny)
import(svglite)
importFrom("grDevices", "dev.copy", "dev.interactive", "dev.new", "dev.off", "dev.size", "png")
importFrom("grDevices", "dev.list")
importFrom("grDevices", "pdf")
importFrom("graphics", "par")
importFrom("htmltools", "htmlDependency")
importFrom("jsonlite", "fromJSON", "toJSON")
importFrom("kableExtra", "scroll_box", "kable_styling", "kbl", "column_spec", "row_spec")
importFrom("stats", "coef", "kmeans", "lm")
importFrom("stats", "rnorm")
importFrom("utils", "install.packages")
importFrom("utils", "packageDescription", "packageVersion")
importFrom("utils", "write.csv")
importFrom("utils", "write.table")

