---
title: "The InteractiveComplexHeatmap package"
author: "Zuguang Gu (z.gu@dkfz.de)"
date: '`r Sys.Date()`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The InteractiveComplexHeatmap package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Please go to the package's website: https://jokergoo.github.io/InteractiveComplexHeatmap.
