---
title: "InteractiveComplexHeatmap in an R Markdown document"
author: "Zuguang Gu"
date: "01/01/2021"
output: html_document
runtime: shiny
---

This document demostrates an interactive heatmap.

```{r, echo = FALSE}
library(ComplexHeatmap)
library(InteractiveComplexHeatmap)
m = matrix(rnorm(100*100), 100)
ht = Heatmap(m)

htShiny(ht)
```


The source code of this R Markdown document is:

````markdown
---
title: "InteractiveComplexHeatmap in an Rmarkdown document"
author: "Zuguang Gu"
date: "16/12/2020"
output: html_document
runtime: shiny
---

This document demostrates an interactive heatmap.

`r ''````{r, echo=FALSE}
library(ComplexHeatmap)
library(InteractiveComplexHeatmap)
m = matrix(rnorm(100*100), 100)
ht = Heatmap(m)

htShiny(ht)
```
````
